--
-- Clean Player Frame
--
-- by Bastian Pflieger <wb@illogical.de>
--
-- Last update: August 28, 2006
--
-- supports "myAddOns": http://www.curse-gaming.com/mod.php?addid=358
--

-- CleanPlayerFrameCombo Modes:
-- 1 - Disabled
-- 2 - Hide text
-- 3 - Show only current value

-- CleanPlayerPartyHealth Modes:
-- 1 - Disabled
-- 2 - Show numbersm
-- 3 - Show colorized numbers
-- 4 - Show colorized numbers and heal assist messages

local TextStatusBar_UpdateTextString_Org;
local PERCENT_MESSAGE_WARN = 45;
local PERCENT_MESSAGE_DANGER = 30;

function CleanPlayerFrame_OnLoad()
  CleanPlayerFrame_Mode = 2; -- defaults to hide text
  CleanPlayerFrame_ShowPartyHealth = 2; -- defaults to show party health
  CleanPlayerFrame_ColorizeHealthBars = true;
  CleanPlayerFrame_ShowTargetInfo = true;
  CleanPlayerFrame_ShowPartyPercent = false;
  CleanPlayerFrame_UseSmallFont = false;

  this:RegisterEvent("VARIABLES_LOADED");
  this:RegisterEvent("PARTY_MEMBERS_CHANGED");

  TextStatusBar_UpdateTextString_Org = TextStatusBar_UpdateTextString;
  TextStatusBar_UpdateTextString = CleanPlayerFrame_TextStatusBar_UpdateTextString;

  SlashCmdList["CLEANPLAYERFRAME"] = function(msg)
    ShowUIPanel(CleanPlayerFrameOptionsFrame);
  end
  SLASH_CLEANPLAYERFRAME1 = "/cleanplayerframe";

  HealthBar_OnValueChanged_Org = HealthBar_OnValueChanged;
  HealthBar_OnValueChanged = CleanPlayerFrame_HealthBar_OnValueChanged;

  TargetFrameManaBar:SetScript("OnValueChanged", CleanPlayerFrame_TextStatusBar_UpdateTextString);

  ChatFrame1:AddMessage("Cleanplayer loaded");
end


function CleanPlayerFrame_Setup(mode, partyHealth, colorizeHealthBars, showTargetInfo, showPartyPercent, useSmallFont)

  CleanPlayerFrame_Mode = mode;
  if mode > 1 then
    ManaBarColor[0].prefix = "";
    ManaBarColor[1].prefix = "";
    ManaBarColor[2].prefix = "";
    ManaBarColor[3].prefix = "";
    SetTextStatusBarTextPrefix(PlayerFrameHealthBar, "");
  else
    ManaBarColor[0].prefix = MANA;
    ManaBarColor[1].prefix = RAGE_POINTS;
    ManaBarColor[2].prefix = FOCUS_POINTS;
    ManaBarColor[3].prefix = ENERGY_POINTS;
    SetTextStatusBarTextPrefix(PlayerFrameHealthBar, HEALTH);
  end
  SetTextStatusBarTextPrefix(PlayerFrameManaBar, ManaBarColor[UnitPowerType("player")].prefix);
  TextStatusBar_UpdateTextString(PlayerFrameHealthBar);
  TextStatusBar_UpdateTextString(PlayerFrameManaBar);
  TextStatusBar_UpdateTextString(PetFrameHealthBar);
  TextStatusBar_UpdateTextString(PetFrameManaBar);

  CleanPlayerFrame_ShowPartyHealth = partyHealth;
  if not (CleanPlayerFrame_ShowPartyHealth == 1) then
    PartyMemberFrame1HealthBar.TextString = CleanPlayerFrame_PartyMemberFrame1HealthBarText;
    PartyMemberFrame2HealthBar.TextString = CleanPlayerFrame_PartyMemberFrame2HealthBarText;
    PartyMemberFrame3HealthBar.TextString = CleanPlayerFrame_PartyMemberFrame3HealthBarText;
    PartyMemberFrame4HealthBar.TextString = CleanPlayerFrame_PartyMemberFrame4HealthBarText;
    PartyMemberFrame1HealthBar.lockShow = 1;
    PartyMemberFrame2HealthBar.lockShow = 1;
    PartyMemberFrame3HealthBar.lockShow = 1;
    PartyMemberFrame4HealthBar.lockShow = 1;
  else
    PartyMemberFrame1HealthBar.TextString = nil;
    PartyMemberFrame2HealthBar.TextString = nil;
    PartyMemberFrame3HealthBar.TextString = nil;
    PartyMemberFrame4HealthBar.TextString = nil;
    -- just to make sure
    CleanPlayerFrame_PartyMemberFrame1HealthBarText:Hide();
    CleanPlayerFrame_PartyMemberFrame2HealthBarText:Hide();
    CleanPlayerFrame_PartyMemberFrame3HealthBarText:Hide();
    CleanPlayerFrame_PartyMemberFrame4HealthBarText:Hide();
  end
  TextStatusBar_UpdateTextString(PartyMemberFrame1HealthBar);
  TextStatusBar_UpdateTextString(PartyMemberFrame2HealthBar);
  TextStatusBar_UpdateTextString(PartyMemberFrame3HealthBar);
  TextStatusBar_UpdateTextString(PartyMemberFrame4HealthBar);

  CleanPlayerFrame_ColorizeHealthBars = colorizeHealthBars;

  CleanPlayerFrame_ShowTargetInfo = showTargetInfo;
  if showTargetInfo then
    TargetFrameHealthBar.TextString = CleanPlayerFrame_TargetFrameHealthBarText;
    TargetFrameManaBar.TextString = CleanPlayerFrame_TargetFrameManaBarText;
    TargetFrameHealthBar.lockShow = 1;
    TargetFrameManaBar.lockShow = 1;
  else
    TargetFrameHealthBar.TextString = nil;
    TargetFrameManaBar.TextString = nil;
    CleanPlayerFrame_TargetFrameHealthBarText:Hide();
    CleanPlayerFrame_TargetFrameManaBarText:Hide();
  end
  TextStatusBar_UpdateTextString(TargetFrameHealthBar);
  TextStatusBar_UpdateTextString(TargetFrameManaBar);

  CleanPlayerFrame_ShowPartyPercent = showPartyPercent;
  if showPartyPercent then
    CleanPlayerFrame_PartyMember1HealthBarText:Show();
    CleanPlayerFrame_PartyMember2HealthBarText:Show();
    CleanPlayerFrame_PartyMember3HealthBarText:Show();
    CleanPlayerFrame_PartyMember4HealthBarText:Show();
  else
    CleanPlayerFrame_PartyMember1HealthBarText:Hide();
    CleanPlayerFrame_PartyMember2HealthBarText:Hide();
    CleanPlayerFrame_PartyMember3HealthBarText:Hide();
    CleanPlayerFrame_PartyMember4HealthBarText:Hide();
  end
  TextStatusBar_UpdateTextString(PartyMemberFrame1HealthBar);
  TextStatusBar_UpdateTextString(PartyMemberFrame2HealthBar);
  TextStatusBar_UpdateTextString(PartyMemberFrame3HealthBar);
  TextStatusBar_UpdateTextString(PartyMemberFrame4HealthBar);

  -- use small font for player frame
  CleanPlayerFrame_UseSmallFont = useSmallFont;
  if useSmallFont then
    PlayerFrameHealthBarText:SetFont(CleanPlayerFrame_SmallNumber:GetFont());
    PlayerFrameManaBarText:SetFont(CleanPlayerFrame_SmallNumber:GetFont());
  else
    PlayerFrameHealthBarText:SetFont(TextStatusBarText:GetFont());
    PlayerFrameManaBarText:SetFont(TextStatusBarText:GetFont());
  end
end

function CleanPlayerFrame_TextStatusBar_UpdateTextString(textStatusBar)
  TextStatusBar_UpdateTextString_Org(textStatusBar);

  if not textStatusBar then
    textStatusBar = this;
  end

  if textStatusBar.TextString == nil then
    return;
  end

  local value = textStatusBar:GetValue();
  local _, max = textStatusBar:GetMinMaxValues();
  local percent = CleanPlayerFrame_Percent(value, max);

  if CleanPlayerFrame_Mode == 3 then
    -- handle player / pet frame
    if textStatusBar == PlayerFrameHealthBar or textStatusBar == PlayerFrameManaBar
       or textStatusBar == PetFrameHealthBar or textStatusBar == PetFrameManaBar
    then
      textStatusBar.TextString:SetText(value);
    end
    -- handle party frames
    if ( textStatusBar == PartyMemberFrame1HealthBar
         or textStatusBar == PartyMemberFrame2HealthBar
         or textStatusBar == PartyMemberFrame3HealthBar
         or textStatusBar == PartyMemberFrame4HealthBar )
    then
      textStatusBar.TextString:SetText(value);
    end
  end

  --textStatusBar.TextString:GetText()
  if ( textStatusBar == PartyMemberFrame1HealthBar
       or textStatusBar == PartyMemberFrame2HealthBar
       or textStatusBar == PartyMemberFrame3HealthBar
       or textStatusBar == PartyMemberFrame4HealthBar )
  then
    -- who r we?
    local id = textStatusBar:GetParent():GetID();
    local unit = "party" .. id;
    -- show percent if necessary
    if CleanPlayerFrame_ShowPartyPercent then
      getglobal("CleanPlayerFrame_PartyMember" .. id .. "HealthBarText"):SetText(percent .. "%");
    end

    -- colorize text if required
    if CleanPlayerFrame_ShowPartyHealth >= 3 then
      textStatusBar.TextString:SetTextColor(CleanPlayerFrame_Colorize(percent));
    else
      textStatusBar.TextString:SetTextColor(1, 1, 1);
    end

    -- add assist messages
    if CleanPlayerFrame_ShowPartyHealth == 4 and (not UnitIsDead(unit)) and (not UnitIsGhost(unit)) and UnitIsConnected(unit) then
      local _, class = UnitClass("player");
      if percent < CLEANPLAYERFRAME_MESSAGES[class].percentDanger then
        textStatusBar.TextString:SetText(textStatusBar.TextString:GetText() .. CLEANPLAYERFRAME_MESSAGES[class].danger);
      elseif percent < CLEANPLAYERFRAME_MESSAGES[class].percentWarn then
        textStatusBar.TextString:SetText(textStatusBar.TextString:GetText() .. CLEANPLAYERFRAME_MESSAGES[class].warn);
      end
    end

    --
    if (not UnitIsConnected(unit)) or UnitIsGhost(unit) then
      textStatusBar.TextString:SetTextColor(0.5, 0.5, 0.5);
    elseif UnitIsDead(unit) then
      textStatusBar.TextString:SetTextColor(1, 0.3, 0.3);
    end
  end

  -- handle target frame
  if textStatusBar == TargetFrameHealthBar then
    -- automatically decide to show percent on target frame
    local min, max = textStatusBar:GetMinMaxValues();
    if max == 100 then -- not so good
      textStatusBar.TextString:SetText(value .. "%");
    else
      textStatusBar.TextString:SetText(value .. " / " .. percent .. "%");
    end
    -- Hide when wow shows "dead" over the target health bar
    if UnitIsDead("target") then
      textStatusBar.TextString:Hide();
    else
      textStatusBar.TextString:Show();
    end
  end
  -- for rage and energy always only current value
  if textStatusBar == TargetFrameManaBar then
    if value > 0 then
      if UnitPowerType("target") == 0  then
        textStatusBar.TextString:SetText(value .. " / " .. percent .. "%");
      else --UnitPowerType("target") == 1 or UnitPowerType("target") == 3
        textStatusBar.TextString:SetText(value);
      end
    else
      textStatusBar.TextString:SetText("");
    end
  end
end

function CleanPlayerFrame_HealthBar_OnValueChanged(value, smooth)
  if CleanPlayerFrame_ColorizeHealthBars then
    HealthBar_OnValueChanged_Org(value, true);
  else
    HealthBar_OnValueChanged_Org(value, smooth);
  end
end

function CleanPlayerFrame_Percent(value, maxValue)
  if maxValue == 0 then
    return 0;
  else
    return math.floor(value * 100 / maxValue);
  end
end

function CleanPlayerFrame_Colorize(percent)
  local r = 0;
  local g = 0;

  if percent >= 50 then
    -- red must get up to 255
    g = 1;
    r = (100 - percent) / 50;
  else
    -- green must get down to 0
    r = 1;
    g = percent / 50;
  end

  return r, g, 0;
end

function CleanPlayerFrame_OnEvent(event)
  -- party members changed && variables loaded
  CleanPlayerFrame_Setup(CleanPlayerFrame_Mode,
                         CleanPlayerFrame_ShowPartyHealth,
                         CleanPlayerFrame_ColorizeHealthBars,
                         CleanPlayerFrame_ShowTargetInfo,
                         CleanPlayerFrame_ShowPartyPercent,
                         CleanPlayerFrame_UseSmallFont);
  -- myaddons reg
  if event == "VARIABLES_LOADED" then
    -- Convert old CleanPlayerFrame_ShowPartyHealth bool variable into integer
    if type(CleanPlayerFrame_ShowPartyHealth) == "boolean" and  CleanPlayerFrame_ShowPartyHealth == true  then
      CleanPlayerFrame_ShowPartyHealth = 2;
      ChatFrame1:AddMessage("Converting bool to 2");
    elseif type(CleanPlayerFrame_ShowPartyHealth) == "boolean" and  CleanPlayerFrame_ShowPartyHealth == false then
      CleanPlayerFrame_ShowPartyHealth = 1;
      ChatFrame1:AddMessage("Converting bool to 1");
    end
    --
    CleanPlayerFrame_Setup(CleanPlayerFrame_Mode,
                           CleanPlayerFrame_ShowPartyHealth,
                           CleanPlayerFrame_ColorizeHealthBars,
                           CleanPlayerFrame_ShowTargetInfo,
                           CleanPlayerFrame_ShowPartyPercent,
                           CleanPlayerFrame_UseSmallFont);
    if myAddOnsFrame_Register then
      myAddOnsFrame_Register( {
        name = CLEANPLAYERFRAME_NAME,
        description = CLEANPLAYERFRAME_MYADDONS_DESCRIPTION,
        version = CLEANPLAYERFRAME_VERSION,
        releaseDate = CLEANPLAYERFRAME_MYADDONS_RELEASEDATE,
        author = "wbb",
        email = "<wb@illogical.de>",
        website = "http://www.illogical.de",
        category = MYADDONS_CATEGORY_COMBAT,
        frame = "CleanPlayerFrameFrame",
        optionsframe = "CleanPlayerFrameOptionsFrame"
      }, {
        CLEANPLAYERFRAME_CHATHELP1
      } );
    end
  end
end